import autosize from 'autosize'

function limit(Vue, el, binding, vnode) {
    vnode.context.$emit('input', el.value.slice(0, binding.value))                
    Vue.nextTick(() => {
        autosize.destroy(el);
        autosize(el)
    })
}

export default function install(Vue, options) {
    Vue.directive('focus', {
        inserted(el, binding) {
            if (binding.value === false) return            

            el.focus()
        }
    })

    Vue.directive('alpha-dash', {
        inserted(el, binding, vnode) {
            if (binding.value === false) return

            el.addEventListener('keyup', e => {
                el.value = el.value.replace(/\s+/g, '-')
            })

            el.addEventListener('focus', e => {
                el.value = el.value.replace(/\s+/g, '-')
            })

            el.addEventListener('change', e => {
                el.value = el.value.replace(/\s+/g, '-')
            })
        }
    })

    Vue.directive('lowercase', {
        inserted(el, binding, vnode) {
            if (binding.value === false) return

            el.addEventListener('keyup', e => {
                el.value = el.value.toLowerCase()
            })

            el.addEventListener('change', e => {
                el.value = el.value.toLowerCase()
            })
        }
    })

    Vue.directive('titlecase', {
        inserted(el, binding, vnode) {
            if (binding.value === false) return            

            el.addEventListener('keyup', e => {
                if (e.ctrlKey && e.keyCode == 86) {
                    vnode.context.$emit('input', el.value.replace(
                        /\w\S*/g,
                        function(txt) {
                            return txt.charAt(0).toUpperCase() + txt.substr(1).toLowerCase();
                        }
                    ));
                }
            })
        }
    })

    Vue.directive('autosize', {        
        update(el, binding) {
            if (binding.value === false) {
                return
            }

            Vue.nextTick(() => {
                autosize.destroy(el);
                autosize(el)
            })
        }
    })

    Vue.directive('limit', {        
        inserted(el, binding, vnode) {
            if (!binding.value) {
                return
            }

            el.addEventListener('keydown', e => {
                let allowedKeyCodes = [46, 8, 37, 39, 86, 65, 67];
                if (
                    (
                        !allowedKeyCodes.includes(e.keyCode) || 
                        (e.keyCode == 86 && !e.ctrlKey) || 
                        (e.keyCode == 65 && !e.ctrlKey) ||
                        (e.keyCode == 67 && !e.ctrlKey)
                    ) && 
                    el.value.length >= binding.value                    
                ) {
                    e.preventDefault()
                }
            })

            el.addEventListener('keyup', e => {
                if (e.keyCode !== 17 && !e.ctrlKey) {
                    limit(Vue, el, binding, vnode)
                }
            })

            el.addEventListener('change', e => {
                if (e.keyCode !== 17 && !e.ctrlKey) {
                    limit(Vue, el, binding, vnode)
                }
            })
        }
    })
}